#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
#= コマンドライン引数の型チェックと設定を扱うクラス
#
# コマンドライン引数は"key=value"、"-key"のいずれかの形式を想定している。
# Unix系OSの引数の標準形式とは異なることに注意する。
#
# チェック内容:
#   1. 指定可能な引数を指定し、それ以外の引数がコマンドラインで指定されればエラー終了(エラーをraise)する。
#   2. 必須の引数を指定し、コマンドラインで指定されていなければエラー終了させる。
#   3. 引数の型を設定し、型に応じた値チェック及び変換をおこなう。
#
# 扱える型:
#   rubyの原始型として:
#      1.str: rubyのString型に変換: デフォルト値の指定 
#      2.int: rubyのFixnum型に変換: デフォルト値の指定、有効な範囲チェック
#      3.float: rubyのFloat型に変換: デフォルト値の指定、有効な範囲チェック
#      4.bool: rubyのtrue/falseに変換: -key タイプのオプションが指定されたかどうかの真偽判定
#   特殊な型として:
#      5.file: rubyのString型に変換: ファイルのreadable or writableチェック
#
#=== 例1
# コマンドライン
# ------------------------------------------------------
# $ ruby test.rb v=0.2 -w
# ------------------------------------------------------
#
# test.rbの内容
# ------------------------------------------------------
# require 'nysol/mcmd'
# include MCMD
#
# args=Margs.new(ARGV)
# val  = args.float("v=") # -> 0.2
# flag = args.bool("-w") # -> true
# ------------------------------------------------------
#
#=== 例2 引数存在チェックや型チェックの例
# コマンドライン
# ------------------------------------------------------
# $ ruby test.rb i=dat.csv v=value -abc
	# ------------------------------------------------------
#
# test.rbの内容
# ------------------------------------------------------
# require 'nysol/mcmd'
# include MCMD
#
# # "i=,o=,w=,-flag,-x"以外の引数が指定されればエラー終了する。
# # "i=,w="引数を指定しなければエラー終了する。
# args=Margs.new(ARGV, "i=,o=,w=,-flag,-x", "i=,w=")
# iFileName = args.file("i=") # -> "dat.csv"
# oFileName = args.str("o=","result.csv") # -> "result.csv"
# weight    = args.float("w=",0.1,0.0,1.0) # -> 0.1
# flag      = args.bool("-abc") # -> true
# wFlag     = args.bool("-w") # -> false
# ------------------------------------------------------
import os
import re
class Margs(object): #みなおすこと

	#== コンストラクタ
	# argv: rubyのARGV変数
	#
	# allKeyWords: key=もしくは-keyによる引数キーワードリスト(String Array)
	#   ここで指定した以外の引数がARGVに指定されていないことをチェックし、指定されていればエラー終了する。
	#   keyListを省略した場合はこのチェックをしない。
	#
	# mandatoryKeyWords: key=による引数キーワードリスト(String Array)
	#   ここで指定した引数がコマンドラインで指定されていなければエラー終了する。
	#   mandatoryKeyWordsを省略した場合はこのチェックをしない。
	def __init__(self,argv, allKeyWords=None, mandatoryKeyWords=None, help_func=None,ver_func=None):
		self.argv=argv
		self.allKeyWords=allKeyWords
		self.mandatoryKeyWords=mandatoryKeyWords
		self.keyValue = {}
		self.cmdName = argv[0]

		# コマンドラインで指定された引数を一旦全てHashに格納する。
		for arg in argv[1:]:
			if arg.startswith('-') :
				self.keyValue[arg]=True
				if arg=="--help":
					if help_func :
						help_func()
						exit()
				elif arg=="--version":
					if ver_func:
						ver_func()
						exit()
			else:
				kv =arg.split("=", 1)
				if len(kv) < 1 :
					raise Exception("invalid argument: '%s'"%(arg))

				self.keyValue[kv[0]+"="]=kv[1]

		# allKeyWordsのオプションタイプのキーワードを登録する
		if self.allKeyWords!=None :
			kwlist = self.allKeyWords.split(",")
			for kw in kwlist:
				if kw.startswith('-') and kw not in self.keyValue :
					self.keyValue[kw]=False

			# 指定のキーワード以外のキーワードが指定されていないかチェック
			for kw in self.keyValue.keys():
				if kw not in kwlist:
					raise Exception("I don't know such a argument: '%s'"%(kw))

		# 必須引数のチェック #おかしいような気がする
		if self.mandatoryKeyWords !=None :
			for kw in self.mandatoryKeyWords.split(','):
				if kw not in self.keyValue and not kw.startswith('-'):
					raise Exception("argument '%s' is mandatory"%(kw))


	#== String型引数の値のチェックと取得
	# 返値: 引数で指定された値(String)
	#
	# key: "key="形式の引数キーワード(String)
	#   ここで指定した引数の値をStringとして返す。
	#   コマンドラインで指定されていなければdefaultの値を返す。
	#
	# default: コマンドラインで指定されなかったときのデフォルト値(String)
	def str(self,key,default=None,token1=None,token2=None):
		if key in self.keyValue :
			val = self.keyValue[key]
		else :
			val=default 
			return val

		if val!=None :
			if token1!= None :
				val1=val.split(token1)
				if token2!=None :
					val2=[]
					for v in val1 :
						val2.apend(v.split(token2))
					val = val2
				else:
					val = val1

		return val


	#== Fixnum型引数の値のチェックと取得
	# 返値: 引数で指定された値(Fixnum)
	#
	# key: "key="形式の引数キーワード
	#   ここで指定した引数の値をFloatとして返す。
	#   コマンドラインで指定されていなければdefaultの値を返す。
	#
	# default: コマンドラインで指定されなかったときのデフォルト値
	#
	# from: 値の下限値。指定した値が下限値を下回ればエラー終了する。
	#
	# to: 値の上限値。指定した値が上限値を上回ればエラー終了する。
	def int(self,key, default=None, fr=None, to=None):

		if key in self.keyValue :
			val = self.keyValue[key]
		else :
			val=default 

		if val != None :
			val=int(val)
			if ( fr != None and val<fr ) or ( to != None and val>to ):
				raise Exception("range error: '%s=%s': must be in [%s..%s]"%(key,val,fr,to))

		return val

	#== Float型引数の値のチェックと取得
	# 返値: 引数で指定された値(Float)
	#
	# key: "key="形式の引数キーワード
	#   ここで指定した引数の値をFloatとして返す。
	#   コマンドラインで指定されていなければdefaultの値を返す。
	#
	# default: コマンドラインで指定されなかったときのデフォルト値
	#
	# from: 値の下限値。指定した値が下限値を下回ればエラー終了する。
	#
	# to: 値の上限値。指定した値が上限値を上回ればエラー終了する。
	def float(self,key, default=None, fr=None, to=None):
		if key in self.keyValue :
			val = self.keyValue[key]
		else :
			val=default 

		if val!= None :
			val=float(val)
			if ( fr != None and val < fr ) or (to != None and val>to) :
				raise Exception("range error: '%s=%s': must be in [%s..%s]"%(key,val,fr,to))

		return val

	#== Bool型引数の値のチェックと取得
	# 返値: 引数で指定されたかどうか(true/false)
	#
	# key: "-key"形式の引数キーワード
	#   ここで指定した引数がコマンドラインで指定されていればtrueを、指定されていなければfalseを返す。
	def bool(self,key):
		if key in self.keyValue :
			return self.keyValue[key]
		else :
			return False

	#== ファイル型引数の値のチェックと取得
	# 返値: 引数で指定されたファイル名(String)
	#
	# key: "key="形式の引数キーワード
	#   ここで指定した引数の値をファイル名と想定し、そのファイルがreadable(writable)かどうかをチェックする。
	#   readable(writable)であればそのファイル名を返し、readable(writable)でなければエラー終了する。
	#   readable(writable)チェックをしないのであればMargs::strを使えばよい。
	#
	# mode: "r"もしくは"w"を指定し、rならばreadableチェックを、wならwritebleチェックを行う。
	def file(self,key,mode="r",default=None):

		if key in self.keyValue :
			val = self.keyValue[key]
		else :
			val=default 

		# valがnilの場合(ex. 値なし指定"i=")はノーチェックで通す?
		if val!=None :
			if mode=="r" :
				if not os.access(val, os.R_OK) :
					raise Exception("file open error: '%s' is not readable"%(val))

			elif mode=="w" :
				if os.path.dirname(val) == "":
					if not os.access(".", os.W_OK) :
						raise Exception("file open error: '%s' is not writable"%(val))				
				else:
					if not os.access(os.path.dirname(val), os.W_OK) :
						raise Exception("file open error: '%s' is not writable"%(val))

		return val


	#== Field型引数の値のチェックと取得
	# 返値: 各種配列のHash
	# 	key=a1:b1%c1,a2:b2%c2,...
	#   names: [a1,a2,...]
	#   newNames: [b1,b2,...]
	#   flags:  [c1,c2,...]
	#   fld2csv: a1,a2,...のCSVファイルにおける項目番号(0から始まる)
	#   csv2fld: CSVファイルの項目番号に対するa1,a2,...の番号(0から始まる)
	# PIPEつかえなくなる
	def field(self,key,iFile,default=None,min=None,max=None) :
		if not iFile :
			return None

		if key in self.keyValue :
			val = self.keyValue[key]
		else :
			val=default 

		names1=[]
		names2=[]
		flags=[]
		fld2csv=[]
		csv2fld=[]

		# names1,names2,flagsの設定
		if val!=None :
			val1=val.split(",")
			for v in val1 :
				val2=v.split("%")
				if len(val2) == 1:
					val2.append(None) 
				val3=val2[0].split(":")
				if len(val3) == 1:
					val3.append(None) 
				names1.append(val3[0])
				names2.append(val3[1])
				flags.append(val2[1])

			if min != None and len(names1)<min :
				raise Exception("%s takes at least %s field name(s)"%(key,min) )

			if max != None and len(names1)>max :
				raise Exception("%s takes at most %s field name(s)"%(key,max) )

			#dmy
			iNames=None
			"""
			iNames=MCMD::Mcsvin.new("i=#{iFile}").names
			# fld2csvの設定
			(0...names1.size).each{|i|
				pos=iNames.index(names1[i])
				if pos==nil then
					raise "field name not found: `#{names1[i]}'"
				end
				fld2csv << pos
			}

			# csv2fldの設定
			(0...iNames.size).each{|i|
				pos=fld2csv.index(i)
				if pos!=nil
					csv2fld << pos
				else
					csv2fld << nil
				end
			}
			"""

			ret={}
			ret["names"]=names1
			ret["newNames"]=names2
			ret["flags"]=flags
			ret["fld2csv"]=fld2csv
			ret["csv2fld"]=csv2fld
			ret["csvNames"]=iNames
			return ret
		else:
			return None


	# key-valuを配列で返す
	def getKeyValue(self,prefix=""):
		ret=[]
		for k,v in self.keyValue.items():
			ret.append([prefix+k,str(v)])
		return ret

	def kvmap(self):
		ret={}
		for k,v in self.keyValue.items():
			newk = re.sub(r'^-',"" ,re.sub(r'=$',"",k))
			ret[newk] =v
		return ret

	# コマンドラインをkey-valuを配列で返す
	def cmdline(self):
		return "%s %s"%(self.cmdName,' '.join(self.argv))

#==============================================
#以下、サンプルコード(require時は実行されない)
#==============================================
if __name__ == '__main__':

	# $ ruby ./margs.rb i=dat.csv v=value -abc
	import sys
	args=Margs( sys.argv, "i=,o=,w=,v=,-flag,-x,-abc", "i=,w=")


	#	args=Margs(args, "f=")
	#	fld=args.field("f=","xxa")

#	print(fld["names"])
#	print(fld["newNames"])
#	print(fld["flags"])
#	print(fld["fld2csv"])
#	print(fld["csv2fld"])

	#p args.str("f=",",",":")
	iFileName = args.file("i=") # -> "dat.csv"
	oFileName = args.str("o=","result.csv") # -> "result.csv"
	weight    = args.float("w=",0.1,0.0,1.0) # -> 0.1
	flag      = args.bool("-abc") # -> true
	wFlag     = args.bool("-w") # -> false




# -*- coding: utf-8 -*-
import copy
import os
import nysol._nysolshell_core as n_core
# 仮 kgshellへ移行
import psutil as ps

class LineListIter(object):

	def __init__(self,obj,dtype=None,skeys=None,q=False,header=False):

		if len(obj.outlist["o"])!=0:
			raise Exception("Do not specify last output ")
			return None

		if obj.name=="writelist":
			raise Exception("Do not suport writelist ")
			return None

		dupobj = obj.modelcopy()

		from nysol.mcmd.submod.msortchk import Nysol_Msortchk as msortchk
		if skeys != None:
			runobj = msortchk({"k":skeys,"q":q}).addPre(dupobj)
		else :
			runobj = dupobj

		modlist,iolist,linklist,_ = runobj.makeRunNetwork(True)

		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass
		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		kglogdir = os.environ.get('KG_UsingLogD')
		kglogD = ""
		if kglogdir != None: 
			kglogD = str(kglogD)


		memsize = 2000000000
		try:
			memsize = ps.virtual_memory().total
		except:
			pass

		# kgshell stock
		self.shobj = n_core.init(runobj.getMsgFlg(),runobj.runlimit,memsize,py_msg,kglogD)
		if self.shobj == None:
			raise Exception("can not init shell")
			return None			

		self.csvin = n_core.runiter(self.shobj,modlist,linklist)
		if self.csvin == None:
			n_core.cancel(self.shobj)
			raise Exception("can not run iter")
			return None

		self.header = header
		self.fldname = n_core.fldname(self.csvin)
		self.dptn  = n_core.fldtp(self.csvin,dtype)


	def __nextCore(self):
		if self.header:
			self.header = False
			return self.fldname
		
		line = n_core.getLineList(self.csvin,self.dptn)
		if line: 
			return line

		n_core.close(self.csvin)
		n_core.cancel(self.shobj)
		return
		#raise StopIteration()

	def next(self):

		return self.__nextCore()

	def __next__(self):

		return self.__nextCore()


class LineDictIter(object):

	def __init__(self,obj,dtype=None,skeys=None,q=False):
		

		if len(obj.outlist["o"])!=0:
			raise Exception("Do not specify last output ")
			return None

		if obj.name=="writelist":
			raise Exception("Do not suport writelist ")
			return None

			
		dupobj = obj.modelcopy()

		from nysol.mcmd.submod.msortchk import Nysol_Msortchk as msortchk
		if skeys != None:
			runobj = msortchk({"k":skeys,"q":q}).addPre(dupobj)
		else :
			runobj = dupobj

		modlist,iolist,linklist,_ = runobj.makeRunNetwork(True)

		# kgshell stock
		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass

		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		kglogdir = os.environ.get('KG_UsingLogD')
		kglogD = ""
		if kglogdir != None: 
			kglogD = str(kglogD)

		memsize = 2000000000
		try:
			memsize = ps.virtual_memory().total
		except:
			pass

		self.shobj = n_core.init(runobj.getMsgFlg(),runobj.runlimit,memsize,py_msg,kglogD)
		if self.shobj == None:
			raise Exception("can not init shell")
			return None			

		self.csvin = n_core.runiter(self.shobj,modlist,linklist)
		if self.csvin == None:
			n_core.cancel(self.shobj)
			raise Exception("can not run iter")
			return None

		self.fldname = n_core.fldname(self.csvin)
		self.dptn  = n_core.fldtp(self.csvin,dtype)

	def __nextCore(self):
	
		line = n_core.getLineDict(self.csvin,self.dptn)
		if line: 
			return line

		n_core.close(self.csvin)
		n_core.cancel(self.shobj)
		return
		#raise StopIteration()

	def next(self):

		return self.__nextCore()

	def __next__(self):

		return self.__nextCore()

class BlkListIter(object):

	def __init__(self,obj,keys,skeys=None,dtype=None,q=False,header=False):

		if len(obj.outlist[obj.nowdir])!=0:
			raise Exception("Do not specify last output ")
			return None

		if obj.name=="writelist":
			raise Exception("Do not suport writelist ")
			return None

		if isinstance(keys,str) :
			newkeys = keys.split(",") 
		elif isinstance(keys,list) :
			newkeys = keys
		else:
			raise Exception("unsuport TYPE")

		dupobj = obj.modelcopy()

		from nysol.mcmd.submod.msortchk import Nysol_Msortchk as msortchk
		sortkeys = copy.deepcopy(newkeys)
		if skeys != None:
			if isinstance(keys,str) :
				sortkeys.extend(skeys.split(","))
			elif isinstance(keys,list) :
				sortkeys.extend(skeys)
			else:
				raise Exception("unsuport TYPE")
			
		runobj = msortchk({"k":sortkeys,"q":q}).addPre(dupobj)
				
		modlist,iolist,linklist,_ = runobj.makeRunNetwork(True)

		# kgshell stock
		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass

		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		kglogdir = os.environ.get('KG_UsingLogD')
		kglogD = ""
		if kglogdir != None: 
			kglogD = str(kglogD)

		memsize = 2000000000
		try:
			memsize = ps.virtual_memory().total
		except:
			pass

		self.shobj = n_core.init(runobj.getMsgFlg(),runobj.runlimit,memsize,py_msg,kglogD)
		if self.shobj == None:
			raise Exception("can not init shell")
			return None			

		self.csvin = n_core.runiter(self.shobj,modlist,linklist,newkeys)
		if self.csvin == None:
			n_core.cancel(self.shobj)
			raise Exception("can not run iter")
			return None

		self.header = header
		self.fldname = n_core.fldname(self.csvin)
		self.dptn  = n_core.fldtp(self.csvin,dtype)

	def __nextCore(self):

		if self.header:
			self.header = False
			return [self.fldname]

		line = n_core.getBlkList(self.csvin,self.dptn)
		if line: 
			return line

		n_core.close(self.csvin)
		n_core.cancel(self.shobj)
		return
		#raise StopIteration()

	def next(self):
		
		return self.__nextCore()


	def __next__(self):

		return self.__nextCore()

class BlkDictIter(object):

	def __init__(self,obj,keys,skeys=None,dtype=None,q=False):

		if len(obj.outlist["o"])!=0:
			raise Exception("Do not specify last output ")
			return None

		if obj.name=="writelist":
			raise Exception("Do not suport writelist ")
			return None

		if isinstance(keys,str) :
			newkeys = keys.split(",") 
		elif isinstance(keys,list) :
			newkeys = keys
		else:
			raise Exception("unsuport TYPE")

		dupobj = obj.modelcopy()

		from nysol.mcmd.submod.msortchk import Nysol_Msortchk as msortchk
		sortkeys = copy.deepcopy(newkeys)
		if skeys != None:
			if isinstance(keys,str) :
				sortkeys.extend(skeys.split(","))
			elif isinstance(keys,list) :
				sortkeys.extend(skeys)
			else:
				raise Exception("unsuport TYPE")
			
		runobj = msortchk({"k":sortkeys,"q":q}).addPre(dupobj)

		modlist,iolist,linklist,_ = runobj.makeRunNetwork(True)

		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass

		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		kglogdir = os.environ.get('KG_UsingLogD')
		kglogD = ""
		if kglogdir != None: 
			kglogD = str(kglogD)

		memsize = 2000000000
		try:
			memsize = ps.virtual_memory().total
		except:
			pass
		# kgshell stock
		self.shobj = n_core.init(runobj.getMsgFlg(),runobj.runlimit,memsize,py_msg,kglogD)
		if self.shobj == None:
			raise Exception("can not init shell")
			return None			

		self.csvin = n_core.runiter(self.shobj,modlist,linklist,newkeys)
		if self.csvin == None:
			n_core.cancel(self.shobj)
			raise Exception("can not run iter")
			return None

		self.fldname = n_core.fldname(self.csvin)
		self.dptn  = n_core.fldtp(self.csvin,dtype)

	def __nextCore(self):

		line = n_core.getBlkDict(self.csvin,self.dptn)
		if line: 
			return line

		n_core.close(self.csvin)
		n_core.cancel(self.shobj)
		return
		#raise StopIteration()


	def next(self):

		return self.__nextCore()

	def __next__(self):

		return self.__nextCore()


class LineListIterWithInfo(object):

	def __init__(self,obj,keys,skeys=None,dtype=None,q=False,header=False):

		if len(obj.outlist["o"])!=0:
			raise Exception("Do not specify last output ")
			return None

		if obj.name=="writelist":
			raise Exception("Do not suport writelist ")
			return None

		
		if isinstance(keys,str) :
			newkeys = keys.split(",") 
		elif isinstance(keys,list) :
			newkeys = keys
		else:
			raise Exception("unsuport TYPE")

		
		dupobj = obj.modelcopy()

		from nysol.mcmd.submod.msortchk import Nysol_Msortchk as msortchk
		sortkeys = copy.deepcopy(newkeys)
		if skeys != None:
			if isinstance(keys,str) :
				sortkeys.extend(skeys.split(","))
			elif isinstance(keys,list) :
				sortkeys.extend(skeys)
			else:
				raise Exception("unsuport TYPE")
			
		runobj = msortchk({"k":sortkeys,"q":q}).addPre(dupobj)

				
		modlist,iolist,linklist,_ = runobj.makeRunNetwork(True)

		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass

		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		kglogdir = os.environ.get('KG_UsingLogD')
		kglogD = ""
		if kglogdir != None: 
			kglogD = str(kglogD)

		memsize = 2000000000
		try:
			memsize = ps.virtual_memory().total
		except:
			pass

		# kgshell stock
		self.shobj = n_core.init(runobj.getMsgFlg(),runobj.runlimit,memsize,py_msg,kglogD)
		if self.shobj == None:
			raise Exception("can not init shell")
			return None			

		self.csvin = n_core.runiter(self.shobj,modlist,linklist,newkeys)
		if self.csvin == None:
			n_core.cancel(self.shobj)
			raise Exception("can not run iter")
			return None

		self.header = header

		self.dptn  = n_core.fldtp(self.csvin,dtype)
		self.fldname = n_core.fldname(self.csvin)
		self.breakPre = True

	def __nextCore(self):

		if self.header:
			self.header = False
			return self.fldname,None,None
		
		data = n_core.getLineListWithInfo(self.csvin,self.dptn)
		if data: 
			breakTop = self.breakPre
			self.breakPre = data[1]
			return data[0],breakTop,data[1]

		n_core.close(self.csvin)
		n_core.cancel(self.shobj)
		return
		#raise StopIteration()
	
	
	def next(self):

		return self.__nextCore()

	def __next__(self):

		return self.__nextCore()
		

class LineDictIterWithInfo(object):

	def __init__(self,obj,keys,skeys=None,dtype=None,q=False):

		if len(obj.outlist["o"])!=0:
			raise Exception("Do not specify last output ")
			return None

		if obj.name=="writelist":
			raise Exception("Do not suport writelist ")
			return None

		dupobj = obj.modelcopy()


		if isinstance(keys,str) :
			newkeys = keys.split(",") 
		elif isinstance(keys,list) :
			newkeys = keys
		else:
			raise Exception("unsuport TYPE")

		from nysol.mcmd.submod.msortchk import Nysol_Msortchk as msortchk

		sortkeys = copy.deepcopy(newkeys)
		if skeys != None:
			if isinstance(keys,str) :
				sortkeys.extend(skeys.split(","))
			elif isinstance(keys,list) :
				sortkeys.extend(skeys)
			else:
				raise Exception("unsuport TYPE")

		runobj = msortchk({"k":sortkeys,"q":q}).addPre(dupobj)

		modlist,iolist,linklist,_ = runobj.makeRunNetwork(True)

		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass

		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		kglogdir = os.environ.get('KG_UsingLogD')
		kglogD = ""
		if kglogdir != None: 
			kglogD = str(kglogD)

		memsize = 2000000000
		try:
			memsize = ps.virtual_memory().total
		except:
			pass

		# kgshell stock
		self.shobj = n_core.init(runobj.getMsgFlg(),runobj.runlimit,memsize,py_msg,kglogD)
		if self.shobj == None:
			raise Exception("can not init shell")
			return None			

		self.csvin = n_core.runiter(self.shobj,modlist,linklist,newkeys)
		if self.csvin == None:
			n_core.cancel(self.shobj)
			raise Exception("can not run iter")
			return None
 
		self.fldname = n_core.fldname(self.csvin)

		self.dptn  = n_core.fldtp(self.csvin,dtype)

		self.breakPre = True

	def __nextCore(self):

		data = n_core.getLineDictWithInfo(self.csvin,self.dptn)

		if data: 
			breakTop = self.breakPre
			self.breakPre = data[1]
			return data[0],breakTop,data[1]

		n_core.close(self.csvin)
		n_core.cancel(self.shobj)
		return
		#raise StopIteration()


	
	def next(self):

		return self.__nextCore()


	def __next__(self):

		return self.__nextCore()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
グラフ構造データクラス
"""
__author__  = "Hiroaki Maruhashi <info@nysol.jp>"
__version__ = "1.00"
__date__    = "28 Dec 2017"

import os
import nysol.mcmd as nm
import nysol.util.mtemp as mtemp

class graph(object):

	def __init__(self,edgeFile=None,edgeList=None,title1=None,title2=None,nodeFile=None,nodeList=None,title=None):
		#メンバ変数定義
		self.eFileT		= None # Take内部で使用するエッジファイル(アイテム間をスペース区切りにしたファイル)
		self.eFile		= None # エッジファイル
		self.eFileN		= None # エッジファイル名
		self.edgeFN1	= None # エッジファイル項目名1(String)
		self.edgeFN2	= None # エッジファイル項目名2(String)
		self.nFile		= None # ノードファイル
		self.nFileN		= None # ノードファイル名
		self.nodeFN		= None # ノードファイル項目名(String)
		self.mFile		= None # ノードIDマッピングファイル
		self.__wfE		= None # エッジ用ワークファイル
		self.__wfN		= None # ノード用ワークファイル
		self.__tempW	= mtemp.Mtemp()
		self.__temp		= mtemp.Mtemp()
		#引数指定があればデータ読み込み(ファイルとリストが同時に指定された場合はファイル優先)
		#エッジ読み込み

		if edgeFile is not None:
			self.__readEdgeCSV(edgeFile=edgeFile,title1=title1,title2=title2)
		else:
			if edgeList is not None:
				self.__readEdgeList(edgeList=edgeList,title1=title1,title2=title2)
		#ノード読み込み
		if nodeFile is not None:
			self.__readNodeCSV(nodeFile=nodeFile,title=title)
		else:
			if nodeList is not None:
				self.__readNodeList(nodeList=nodeList,title=title)
		#符号化
		if self.__wfE is not None:
		  self.__convertToNumeric()

	def __readEdgeCSV(self,edgeFile=None,title1=None,title2=None):
		self.eFileN	= edgeFile
		if title1 is None or title2 is None :
			mcmd = nm.mcut(nfni=True,f="0:e1,1:e2",i=self.eFileN)
			self.edgeFN1 = "e1"
			self.edgeFN2 = "e2"
		else :
			self.edgeFN1 = title1
			self.edgeFN2 = title2
			mcmd = nm.mcut(f="%s:e1,%s:e2"%(self.edgeFN1,self.edgeFN2),i=self.eFileN)
		self.__wfE = self.__tempW.file()
		mcmd <<= nm.mnumber(q=True,a="no",o=self.__wfE)
		mcmd.run()

	def __readEdgeList(self,edgeList=None,title1=None,title2=None):
		if title1 is not None and title2 is not None :
			self.edgeFN1 = title1
			self.edgeFN2 = title2
		else:
			self.edgeFN1 = "e1"
			self.edgeFN2 = "e2"
			
		mcmd = nm.mcut(nfni=True,f="0:e1,1:e2",i=edgeList)
		self.__wfE = self.__tempW.file()
		mcmd <<= nm.mnumber(q=True,a="no",o=self.__wfE)
		mcmd.run()

	def __readNodeCSV(self,nodeFile=None,title=None):
		if nodeFile is not None:
			self.nFileN	= nodeFile
			if title is None :
				mcmd = nm.mcut(nfni=True,f="0:n",i=self.nFileN)
			else :
				self.nodeFN = title
				mcmd = nm.mcut(f="%s:n"%(self.nodeFN),i=self.nFileN)
			self.__wfN = self.__tempW.file()
			mcmd <<= nm.mnumber(q=True,a="no",o=self.__wfN)
			mcmd.run()
	
	def __readNodeList(self,nodeList=None,title=None):
		if nodeList is not None:
			if title is not None :
				self.nodeFN = title
			self.__wfN = self.__tempW.file()
			mcmd =   nm.m2cross(nfn=True,a="num,node",f="0-%d"%(len(nodeList)-1),i=[nodeList])
			mcmd <<= nm.mcut(nfni=True,f="1:n,0:no",o=self.__wfN)
			mcmd.run()
	
	def __convertToNumeric(self):
		wf1 = self.__tempW.file()
		wf2 = self.__tempW.file()
		wf3 = self.__tempW.file()
		self.mFile = self.__temp.file()
		nm.mcut(f="e1:node",i=self.__wfE,o=wf1).run()
		nm.mcut(f="e2:node",i=self.__wfE,o=wf2).run()
		mcmd = None
		if self.__wfN is not None:
			nm.mcut(f="n:node",i=self.__wfN,o=wf3).run()
			mcmd <<= nm.mcat(i=wf1+","+wf2+","+wf3,f="node")
		else:
			mcmd <<= nm.mcat(i=wf1+","+wf2,f="node")
		mcmd <<= nm.muniq(k="node")
		mcmd <<= nm.mnumber(q=True,a="id",o=self.mFile)
		mcmd.run()
		#エッジファイル変換・保存
		self.eFile = self.__temp.file()
		mcmd =   nm.mjoin(i=self.__wfE,m=self.mFile,f="id:id1",k="e1",K="node")
		mcmd <<= nm.mjoin(m=self.mFile,f="id:id2",k="e2",K="node")
		mcmd <<= nm.muniq(k="id1,id2")
		mcmd <<= nm.msortf(f="id1%n,id2%n")
		mcmd <<= nm.mcut(f="id1:e1,id2:e2,no:row_index",o=self.eFile)
		mcmd.run()
		#take内部用エッジファイル保存
		self.eFileT = self.__temp.file()
		mcmd =   nm.mcal(i=self.eFile,c="cat(\" \",$s{e1},$s{e2})",a="edge")
		mcmd <<= nm.mcut(nfno=True,f="edge",o=self.eFileT)
		mcmd.run()
		#ノードファイル変換・保存
		self.nFile = self.__temp.file()
		if self.__wfN is not None:
			mcmd =   nm.mjoin(i=self.__wfN,m=self.mFile,f="id",k="n",K="node")
			mcmd <<= nm.muniq(k="id")
			mcmd <<= nm.msortf(f="id%n")
			mcmd <<= nm.mcut(f="id:n,no:row_index",o=self.nFile)
			mcmd.run()
		else:
			#エッジよりノード生成
			wf4 = self.__tempW.file()
			wf5 = self.__tempW.file()
			nm.mcut(i=self.eFile,f="e1:n",o=wf4).run()
			nm.mcut(i=self.eFile,f="e2:n",o=wf5).run()
			mcmd =   nm.mcat(i="%s,%s"%(wf4,wf5))
			mcmd <<= nm.muniq(k="n")
			mcmd <<= nm.msortf(f="n%n")
			mcmd <<= nm.mnumber(q=True,a="row_index",o=self.nFile)
			mcmd.run()
			
		#ワークファイル削除
		self.__tempW.rm()

	def readCSV(self,edgeFile=None,title1=None,title2=None,nodeFile=None,title=None):
		#グラフデータ読み込み済の場合は何もしない
		if self.eFile is not None:
			return
		#エッジファイル読み込み
		if edgeFile is None:
			raise Exception("edgeFile is mandatory.")
		else:
			self.__readEdgeCSV(edgeFile=edgeFile,title1=title1,title2=title2)
		#ノードファイル読み込み
		self.__readNodeCSV(nodeFile=nodeFile,title=title)
		#符号化
		self.__convertToNumeric()

	def readList(self,edgeList=None,title1=None,title2=None,nodeList=None,title=None):
		#グラフデータ読み込み済の場合は何もしない
		if self.eFile is not None:
			return
		#エッジリスト読み込み
		if edgeList is None:
			raise Exception("edgeList is mandatory.")
		else:
			self.__readEdgeList(edgeList=edgeList,title1=title1,title2=title2)
		#ノードリスト読み込み
		self.__readNodeList(nodeList=nodeList,title=title)
		#符号化
		self.__convertToNumeric()

	def features(self):
		nodeSize = int(nm.mcount(i=self.nFile,a="cnt").mcut(f="cnt").run()[0][0])
		edgeSize = int(nm.mcount(i=self.eFile,a="cnt").mcut(f="cnt").run()[0][0])
		dens=None
		if(nodeSize>1):
			dens=float(edgeSize)/(nodeSize*(nodeSize-1)/2.0)
		return nodeSize,edgeSize,dens

	def isSame(self,targetGraph):
		if os.path.getsize(self.eFile)!= os.path.getsize(targetGraph.eFile):
			return False
		else:
			xxt = mtemp.Mtemp()
			xx = xxt.file()
			os.system("diff -q %s %s > %s"%(self.eFile,targetGraph.eFile,xx))
			if(os.path.getsize(xx) != 0):
				xxt.rm()
				return False
			else:
				xxt.rm()
				return True
	
	def node_ids(self):
		if self.nFile is not None:
			return nm.mcut(i=self.nFile,f="n")
	
	def nodes(self):
		if self.nFile is not None:
			return (nm.mjoin(i=self.nFile,m=self.mFile,k="n",K="id",f="node")
								.msortf(f="row_index%n")
								.mcut(f="node:n"))
	
	def edge_ids(self):
		#return nm.mcat(i=self.eFile).run()
		if self.eFile is not None:
			return nm.mcut(i=self.eFile,f="e1,e2")
	
	def edges(self):
		if self.eFile is not None:
			return (nm.mjoin(i=self.eFile,m=self.mFile,k="e1",K="id",f="node:n1")
								.mjoin(m=self.mFile,k="e2",K="id",f="node:n2")
								.msortf(f="row_index%n")
								.mcut(f="n1:e1,n2:e2"))
	
	def printMap(self):
		if self.mFile is not None:
			for r in nm.mcut(i=self.mFile,f="id,node"):
				print(r)

	def printEdges(self):
		if self.eFile is not None:
			for r in nm.mcut(f="e1,e2",i=self.eFile):
				print(r)

	def printEdgesTake(self):
		if self.eFileT is not None:
			for r in nm.mcut(i=self.eFileT,nfni=True,f="0:edge"):
				print(r)

	def printNodes(self):
		if self.nFile is not None:
			for r in nm.mcut(f="n",i=self.nFile):
				print(r)




# -*- coding: utf-8 -*-
import shlex
import re
import copy

class script(object):

	def __init__(self) :
		self.cmdlist=[]
		self.msgFlg=False

	def add(self,obj,**kwd) :
		self.cmdlist.append([obj,kwd])
		#if self.cmdlist==None:
		#	self.cmdlist = obj
		#else:
		#	obj.inp = self.cmdlist
		#	self.cmdlist = obj

	def msgOn(self):
		self.msgFlg=True

	def msgOff(self):
		self.msgFlg=False

	def show(self) :
		print(self.cmdlist)
		
	def makeNetwork(self,cmdlists):

		conectLIST={}
		#f.w.コマンド毎にする
		ioParams = {"i":0,"m":0,"o":1,"u":1}

		# IO list check
		# f.w.
		# コマンド毎にチェックし方を変えれるようにする 
		# io以外にも拡張 
		for linno ,cmd  in enumerate(cmdlists):
			cmdptn = cmd[0]
			cmdio = cmd[1]
			for k ,v in cmdio.items():
				if not isinstance(v,str):
					continue
				ioTP = ioParams[k]
				if ioTP == 0 :
					if not v in conectLIST:
						print( k + " model err before output")
					conectLIST[v][1].append([k,linno])

				elif ioTP == 1 :
					if not v in conectLIST:
						conectLIST[v] =[[],[]]
					conectLIST[v][0].append([k,linno])

		for key ,val in conectLIST.items():
			if len(val[0]) != 1 or len(val[1]) == 0 :
				print( key + " model err")

		#cmd作成
		newcmdlist = []
		interobj = {}
		newruncmd = None

		for linno ,cmd  in enumerate(cmdlists):
			cmdptn = cmd[0]
			cmdio = cmd[1]

			if newruncmd != None and (not "i" in cmdio or len(cmdio["i"])==0):
					tmpptn  = cmdptn
					tmpptn.inplist["i"].append(newruncmd)
					newruncmd.outlist[newruncmd.nowdir].append(cmdptn)
					newruncmd = tmpptn
			else:
				newruncmd = cmdptn

			for k ,v in cmdio.items():
				if isinstance(v,str):
					ioTP = ioParams[k]
					if ioTP == 0 :
						newruncmd.paraUpdate({k:interobj[v][0]})
						interobj[v][0].outlist[interobj[v][1]].append(newruncmd)
					elif ioTP == 1 :
						interobj[v] = [newruncmd,k]
				else:
					newruncmd.paraUpdate({k:v})

			#writescv ,writelistで終了
			if cmdptn.name == "writecsv" or cmdptn.name=="writelist"  :
				newcmdlist.append(newruncmd)
				newruncmd = None

		return newcmdlist
		
		
	def run(self,**kwd) :

		runlist_org = copy.deepcopy(self.cmdlist)
		runcmds = self.makeNetwork(self.cmdlist)

		nowMsgFlg = self.msgFlg
		if "msg" in kwd:
			if kwd["msg"] == "on" :
				nowMsgFlg=True

		from nysol.mcmd.nysollib.core import runs 

		if nowMsgFlg :
			runs(runcmds,msg="on")
		else:
			runs(runcmds)

		self.cmdlist = runlist_org


	def drawModel(self,fname=None):

		runlist = copy.deepcopy(self.cmdlist)
		runcmds = self.makeNetwork(runlist)

		from nysol.mcmd.nysollib.core import drawModels as drawModels

		drawModels(runcmds,fname)

	def drawModelD3(self,fname=None):

		runlist = copy.deepcopy(self.cmdlist)
		runcmds = self.makeNetwork(runlist)

		from nysol.mcmd.nysollib.core import drawModelsD3 as drawModelsD3

		drawModelsD3(runcmds,fname)


	def modelInfo(self):

		runlist = copy.deepcopy(self.cmdlist)
		runcmds = self.makeNetwork(runlist)

		from nysol.mcmd.nysollib.core import modelInfos as modelInfos

		return modelInfos(runcmds)

	def __lshift__(self,obj):
		self.add(obj)



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_M2cross(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd  = n_core.getparalist("m2cross",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_M2cross,self).__init__("m2cross",nutil.args2dict(args,kw_args,Nysol_M2cross._kwd))


def m2cross(self,*args, **kw_args):
	return Nysol_M2cross(nutil.args2dict(args,kw_args,Nysol_M2cross._kwd)).addPre(self)

setattr(NysolMOD_CORE, "m2cross", m2cross)




from setuptools import setup, find_packages, Extension
import re
import os
import subprocess

args = ['xml2-config','--cflags']
xmlcflg= subprocess.check_output(args).decode().rstrip().split()
args = ['xml2-config','--libs']
xmllibs= subprocess.check_output(args).decode().rstrip().split()

hedears = ['src','src/kgmod','src/mod']

def checkLibRun(cc,fname,paras):
	for para in paras:
		try:
			with open(os.devnull, 'w') as fnull:
				exit_code = subprocess.call(cc + [ fname , "-l"+para],
                                    stdout=fnull, stderr=fnull)
		except OSError :
			exit_code = 1

		if exit_code == 0:
			return para

	return ""

def check_for_boost():

	import sysconfig
	import tempfile
	import shutil
	
	# Create a temporary directory
	tmpdir = tempfile.mkdtemp()
	curdir = os.getcwd()
	os.chdir(tmpdir)

	compiler = os.environ.get('CC', sysconfig.get_config_var('CC'))

	# make sure to use just the compiler name without flags
	compiler = compiler.split()
	filename = 'test.cpp'
	with open(filename,'w') as f :
		f.write("""
			int main ()
			{
				return main ();
			  return 0;
			}
			""")

	boostFLG =[]
	boostLibLists=[
		['boost_thread-mt','boost_thread'],
		['boost_filesystem-mt','boost_filesystem'],
		['boost_regex-mt','boost_regex'],
		['boost_system-mt','boost_system'],
		['boost_date_time-mt','boost_date_time']
	]
	for boostlist in boostLibLists:

		boostflg = checkLibRun(compiler,filename, boostlist)
		if boostflg =="" : 	
			#err
			raise Exception("not find "+ " or ".join(boostlist))

		boostFLG.append(boostflg)

	# Clean up
	os.chdir(curdir)
	shutil.rmtree(tmpdir)

	return boostFLG

import sys
from distutils.util import get_platform
plat_name = get_platform()
plat_specifier = ".%s-%d.%d" % (plat_name, *sys.version_info[:2])
build_temp = os.path.join('build', 'temp' + plat_specifier)

def _get_obj_path(src_path):
	tmp_path = os.path.join(build_temp, src_path)
	filename = os.path.splitext(os.path.basename(tmp_path))[0] + '.o'
	return os.path.join(os.path.dirname(tmp_path), filename)

def exclude_no_need_compile_sources(sources):
	"""
	delete unnecessary source
	by comparing timestamp with one of object file

	return tuple which consists of 
	compiling file names and excluded file names
	"""

	compiling = []
	excluded = []

	for src in sources:
		src_path = os.path.join('src', src)
		src_last_updated_at = os.stat(src_path).st_mtime
		obj_path = _get_obj_path(src_path)

		if os.path.exists(obj_path):
			obj_last_updated_at = os.stat(obj_path).st_mtime
			if src_last_updated_at > obj_last_updated_at:
				compiling.append(src_path)
			else:
				excluded.append(obj_path)
		else:
			compiling.append(src_path)

	return compiling, excluded


nmodLibs = ['pthread']
nmodLibs.extend(check_for_boost())
umodLibs = ['pthread']
umodLibs.extend(check_for_boost())
mmodLibs = ['pthread']
mmodLibs.extend(check_for_boost())


sources_core, extra_objects_core = exclude_no_need_compile_sources([
	'mod/kgsortchk.cpp', 'mod/kg2tee.cpp', 'mod/kgfifo.cpp', 'mod/kgtrafld.cpp',
	'kgexcmd.cpp', 'nysolshell.cpp', 'mod/kg2cat.cpp',
	'kgshell.cpp', 'kgsplitblock.cpp', 'kgmsgincpysys.cpp',
	'kgload.cpp', 'kgpyfunc.cpp',
	'kgmod/kgArgFld.cpp',
	'kgmod/kgArgs.cpp', 'kgmod/kgCSV.cpp',
	'kgmod/kgCSVout.cpp', 'kgmod/kgCSVutils.cpp',
	'kgmod/kgEnv.cpp', 'kgmod/kgError.cpp',
	'kgmod/kgFldBuffer.cpp', 'kgmod/kgFldHash.cpp',
	'kgmod/kgFunction.cpp', 'kgmod/kgMessage.cpp',
	'kgmod/kgMethod.cpp', 'kgmod/kgRange.cpp',
	'kgmod/kgTempfile.cpp', 'kgmod/kgVal.cpp',
	'kgmod/kgWildcard.cpp', 'kgmod/kgmod.cpp',
	'kgmod/kgmodincludesort.cpp', 'kgmod/kgsortf.cpp',
	'mod/kgshuffle.cpp', 'mod/kgunicat.cpp',
	'mod/kgcal.cpp', 'mod/kgcut.cpp', 'mod/kgsum.cpp', 'mod/kgjoin.cpp',
	'mod/kg2cross.cpp', 'mod/kgaccum.cpp', 'mod/kgavg.cpp',
	'mod/kgbest.cpp', 'mod/kgbucket.cpp', 'mod/kgchgnum.cpp',
	'mod/kgchgstr.cpp', 'mod/kgcombi.cpp', 'mod/kgcommon.cpp',
	'mod/kgcount.cpp', 'mod/kgcross.cpp', 'mod/kgdelnull.cpp',
	'mod/kgdformat.cpp', 'mod/kgduprec.cpp', 'mod/kgfldname.cpp',
	'mod/kgfsort.cpp', 'mod/kghashavg.cpp', 'mod/kghashsum.cpp',
	'mod/kgkeybreak.cpp', 'mod/kgmbucket.cpp', 'mod/kgmvavg.cpp',
	'mod/kgmvsim.cpp', 'mod/kgmvstats.cpp', 'mod/kgnewnumber.cpp',
	'mod/kgnewrand.cpp', 'mod/kgnewstr.cpp', 'mod/kgnjoin.cpp',
	'mod/kgnormalize.cpp', 'mod/kgnrcommon.cpp', 'mod/kgnrjoin.cpp',
	'mod/kgnullto.cpp', 'mod/kgnumber.cpp', 'mod/kgpadding.cpp',
	'mod/kgpaste.cpp', 'mod/kgproduct.cpp', 'mod/kgrand.cpp',
	'mod/kgrjoin.cpp', 'mod/kgsed.cpp', 'mod/kgsel.cpp',
	'mod/kgselnum.cpp', 'mod/kgselrand.cpp', 'mod/kgselstr.cpp',
	'mod/kgsetstr.cpp', 'mod/kgshare.cpp', 'mod/kgsim.cpp',
	'mod/kgslide.cpp', 'mod/kgsplit.cpp', 'mod/kgstats.cpp',
	'mod/kgsummary.cpp', 'mod/kgtonull.cpp', 'mod/kgtra.cpp',
	'mod/kgtraflg.cpp', 'mod/kguniq.cpp', 'mod/kgcat.cpp',
	'mod/kgvcat.cpp', 'mod/kgvcommon.cpp', 'mod/kgvcount.cpp',
	'mod/kgvdelim.cpp', 'mod/kgvdelnull.cpp', 'mod/kgvjoin.cpp',
	'mod/kgvnullto.cpp', 'mod/kgvreplace.cpp', 'mod/kgvsort.cpp',
	'mod/kgvuniq.cpp', 'mod/kgwindow.cpp', 'mod/kgsep.cpp', 'mod/kgkmeans.cpp',
	'mod/kgarff2csv.cpp', 'mod/kgtab2csv.cpp', 'mod/kgxml2csv.cpp'
])
module1 = Extension('nysol/_nysolshell_core',
                    sources=sources_core,
					extra_objects=extra_objects_core,
					include_dirs=hedears,
					libraries=nmodLibs,
					extra_compile_args=xmlcflg,
					extra_link_args=xmllibs
					)


sources_lcmmod, extra_objects_lcmmod = exclude_no_need_compile_sources(
	['take/lcmrap.c']
)
lcmmod = Extension('nysol/take/_lcmlib',
                    sources=sources_lcmmod,
					extra_objects=extra_objects_lcmmod,
 					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/lcm']
					)


sources_seqmod, extra_objects_seqmod = exclude_no_need_compile_sources(
	['take/lcmseqrap.c']
)
seqmod = Extension('nysol/take/_lcmseqlib',
                    sources=sources_seqmod,
					extra_objects=extra_objects_seqmod,
 					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/lcmseq']
					)


sources_seqmodzero, extra_objects_seqmodzero = exclude_no_need_compile_sources(
	['take/lcmseq_zerorap.c']
)
seqmodzero = Extension('nysol/take/_lcmseq_zerolib',
                    sources=sources_seqmodzero,
					extra_objects=extra_objects_seqmodzero,
 					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/lcmseq_zero']
					)


sources_lcmtransmod, extra_objects_lcmtransmod = exclude_no_need_compile_sources(
	['take/lcmtransrun.cpp']
)
lcmtransmod = Extension('nysol/take/_lcmtranslib',
                    sources=sources_lcmtransmod,
					extra_objects=extra_objects_lcmtransmod,
 					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod']
					)


sources_mmaketramod, extra_objects_mmaketramod = exclude_no_need_compile_sources([
	'take/mmaketrarun.cpp',
	'take/mmaketra/kgmaketra.cpp',
	'kgmod/kgArgFld.cpp',
	'kgmod/kgArgs.cpp', 'kgmod/kgCSV.cpp',
	'kgmod/kgCSVout.cpp', 'kgmod/kgCSVutils.cpp',
	'kgmod/kgEnv.cpp', 'kgmod/kgError.cpp',
	'kgmod/kgFldBuffer.cpp',
	'kgmod/kgMessage.cpp',
	'kgmod/kgMethod.cpp',
	'kgmod/kgTempfile.cpp', 'kgmod/kgVal.cpp',
	'kgmod/kgWildcard.cpp', 'kgmod/kgmod.cpp',
	'kgmod/kgmodincludesort.cpp', 'kgmod/kgsortf.cpp'              
])
mmaketramod = Extension('nysol/take/_mmaketralib',
						sources=sources_mmaketramod,
						extra_objects=extra_objects_mmaketramod,
						extra_compile_args=['-Wno-format-security', '-Wno-error'],
						include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/mmaketra'],
						libraries=mmodLibs
						)


sources_sspcmod, extra_objects_sspcmod = exclude_no_need_compile_sources(
	['take/sspcrap.c']
)
sspcmod = Extension('nysol/take/_sspclib',
                    sources=sources_sspcmod,
					extra_objects=extra_objects_sspcmod,
					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/sspc']
					)


sources_grhfilmod, extra_objects_grhfilmod = exclude_no_need_compile_sources(
	['take/grhfilrap.c']
)
grhfilmod = Extension('nysol/take/_grhfillib',
                    sources=sources_grhfilmod,
					extra_objects=extra_objects_grhfilmod,
					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/grhfil']
					)


sources_macemod, extra_objects_macemod = exclude_no_need_compile_sources(
	['take/macerap.c']
)
macemod = Extension('nysol/take/_macelib',
                    sources = sources_macemod,
					extra_objects=extra_objects_macemod,
					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/mace']
					)


sources_simsetmod, extra_objects_simsetmod = exclude_no_need_compile_sources(
	['take/simsetrap.c']
)
simsetmod = Extension('nysol/take/_simsetlib',
                    sources=sources_simsetmod,
					extra_objects=extra_objects_simsetmod,
					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/simset']
					)


sources_medsetmod, extra_objects_medsetmod = exclude_no_need_compile_sources(
	['take/medsetrap.c']
)
medsetmod = Extension('nysol/take/_medsetlib',
                    sources=sources_medsetmod,
					extra_objects=extra_objects_medsetmod,
					extra_compile_args=['-Wno-format-security', '-Wno-error'],
					include_dirs=['src', 'src/kgmod', 'src/mod', 'src/take/medset']
					)


sources_utilmod, extra_objects_utilmod = exclude_no_need_compile_sources([
	'util/mmethods.cpp',
	'kgmod/kgArgFld.cpp',
	'kgmod/kgArgs.cpp', 'kgmod/kgCSV.cpp',
	'kgmod/kgCSVout.cpp', 'kgmod/kgCSVutils.cpp',
	'kgmod/kgEnv.cpp', 'kgmod/kgError.cpp',
	'kgmod/kgFldBuffer.cpp',
	'kgmod/kgMessage.cpp',
	'kgmod/kgMethod.cpp',
	'kgmod/kgTempfile.cpp', 'kgmod/kgVal.cpp',
	'kgmod/kgWildcard.cpp', 'kgmod/kgmod.cpp',
	'kgmod/kgmodincludesort.cpp','kgmod/kgsortf.cpp',
	'mod/kgchkcsv.cpp'
])
utilmod = Extension('nysol/util/_utillib',
                    sources=sources_utilmod,
					extra_objects=extra_objects_utilmod,
					include_dirs=['src', 'src/kgmod', 'src/mod'],
					libraries=umodLibs
					)


sources_vsopmod, extra_objects_vsopmod = exclude_no_need_compile_sources([
	'vsop/vsop.cpp',
	'vsop/SAPPOROBDD/src/BDDc/bddc.c',
	'vsop/SAPPOROBDD/src/BDD+/BDD.cc',
	'vsop/SAPPOROBDD/src/BDD+/CtoI.cc',
	'vsop/SAPPOROBDD/src/BDD+/ZBDD.cc',
	'vsop/SAPPOROBDD/src/BDD+/ZBDDDG.cc',
	'vsop/SAPPOROBDD/src/BDDLCM/lcm-vsop.cc',
	'vsop/SAPPOROBDD/app/VSOP/table.cc',
	'vsop/SAPPOROBDD/app/VSOP/print.cc'
])
vsopmod = Extension('nysol/vsop/_vsoplib',
                    sources=sources_vsopmod,
					extra_objects=extra_objects_vsopmod,
					define_macros=[('B_64', None), ('_NO_MAIN_', None)],
					extra_compile_args=['-Wno-format-security', '-Wno-error', '-Wno-narrowing'],
					include_dirs=[
						'src/vsop/SAPPOROBDD/include',
						'src/vsop/SAPPOROBDD/src/BDDLCM/',
						'src/vsop/SAPPOROBDD/app/VSOP'
					]
					)

setup(
	name='nysol',
	packages=[
		'nysol',
		'nysol/util',
		'nysol/mcmd',
		'nysol/mcmd/submod',
		'nysol/mcmd/nysollib',
		'nysol/take',
		'nysol/vsop',
		'nysol/take/lib',
		'nysol/take/lib/base'
	],
	version='0.3.9',
	description='This is nysol tools',
	long_description="""\
NYSOL (read as nee-sol) is a generic name of software tools and project activities designed for supporting big data analysis. The tools are developed from various large-scale data analysis and university research projects. NYSOL consists of two packages so far, namely MCMD and TAKE.

MCMD is a set of methods that are developed for the purpose of high-speed processing of large-scale structured tabular data in Python Lists or CSV file. Each method reads the tabular data from standard input, and carry out a very simple processing method (like sort, sum-up, join, merge, etc.) and write the results to standard output. It achieves a variety of processing functions by connecting individual methods with an inter-thread stream called "pipe", thus the output of each process feeds directly as input to the next one. It is possible to efficiently process a large scale of data with hundred millions row of records on a standard PC.

TAKE (pronunciation is "ta-ke", not common "take", stand for the name of developer) is a library for data mining consisting of a varaety of methods, like itemset mining, sequecial pattern mining, clique enumeration, graph polishing, etc. This package developed at the center led by Professor Takeaki Uno (National Institute of Informatics in Japan). TAKE wrapped his original software in Python.

NYSOL runs in UNIX environment (Linux and Mac OS X, not Windows).
""",
	author='nysol',
	author_email='info@nysol.jp',
	license='AGPL3',
	url='http://www.nysol.jp/',
	classifiers=[ 
		'Development Status :: 4 - Beta',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: GNU Affero General Public License v3',
		'Operating System :: POSIX :: Linux',
		'Operating System :: MacOS :: MacOS X',
		'Programming Language :: C',
		'Programming Language :: C++',
		'Programming Language :: Python :: 3',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Topic :: Scientific/Engineering :: Information Analysis',
		'Topic :: Scientific/Engineering :: Mathematics',
	],
	install_requires=["psutil"],
	scripts=[
		'scripts/take/mfriends.py', 'scripts/take/mitemset.py',
		'scripts/take/mpolishing.py', 'scripts/take/msequence.py','scripts/take/mccomp.py',
		'scripts/take/mtra2gc.py', 'scripts/take/mpal.py','scripts/take/mgdiff.py',
		'scripts/take/mclique.py', "scripts/take/mbipolish.py", "scripts/take/mbiclique.py"
	],
	ext_modules=[
		module1, lcmmod, sspcmod, grhfilmod, macemod, seqmod,
		seqmodzero, lcmtransmod, macemod, simsetmod, medsetmod, vsopmod,
		utilmod, mmaketramod
	]
)
       



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
#require "nysol/seqDB.rb"
import os
import shutil
import nysol.util.mtemp as mtemp
import nysol.mcmd as nm
from nysol.take import extcore as extTake


#========================================================================
# 列挙関数:lcmseq 利用DB:SeqDB
#========================================================================
class LcmSeq(object):

	def __init__(self,db,outtf=True):
		self.size =None
		self.temp=mtemp.Mtemp()
		self.db = db # 入力データベース
		self.file=self.temp.file()
		items=self.db.items
		self.outtf = outtf
		self.top =None
		self.msgoff = True
		# アイテムをシンボルから番号に変換する。
		f = nm.mjoin(k=self.db.itemFN , K=items.itemFN, m=items.file,f=items.idFN,i=self.db.file)
		f <<= nm.mcut(f=self.db.idFN+","+self.db.timeFN+","+items.idFN)
		f <<= nm.mtra(k=self.db.idFN,s=self.db.timeFN+"%n",f=items.idFN)
		f <<= nm.mcut(f=items.idFN,nfno=True,o=self.file)
		f.run()


	def enumerate(self,eArgs):
		tf=mtemp.Mtemp()

		# 最小サポートと最小サポート件数
		if "minCnt" in eArgs and eArgs["minCnt"] != None:
			self.minCnt = int(eArgs["minCnt"])
			self.minSup = float(self.minCnt)/float(self.db.size)
		else:
			self.minSup =float(eArgs["minSup"])
			self.minCnt =int(self.minSup * float(self.db.size) + 0.99)


		# 最大サポートと最大サポート件数
		self.maxCnt=None
		if ("maxCnt" in eArgs and  eArgs["maxCnt"]!= None) or ( "maxSup" in eArgs and eArgs["maxSup"]!= None):
			if "maxCnt" in eArgs and eArgs["maxCnt"]!= None:
				self.maxCnt = int(eArgs["maxCnt"])
				self.maxSup = float(self.maxCnt)/float(self.db.size)
			else:
				self.maxSup = float(eArgs["maxSup"])
				self.maxCnt = int(self.maxSup * float(self.db.size)+ 0.99)


		# 列挙パターン数上限が指定されれば、一度lcmを実行して最小サポートを得る
		if "top" in eArgs and eArgs["top"] != None :
			self.top = eArgs["top"]

		# 列挙パターン数上限が指定されれば、一度lcmを実行して最小サポートを得る
		if self.top and self.top>0 :

			xxtop = tf.file()
			 
			extTake.lcmseq(type="Cf",K=str(self.top),i=self.file,sup="1" ,so=xxtop)

			with open(xxtop, "r") as rfile:
				self.minCnt = int(rfile.read().strip())


		# lcm_seq出力ファイル
		lcmout = tf.file()
		# 頻出パターンがなかった場合、lcm出力ファイルが生成されないので
		# そのときのために空ファイルを生成しておいく。
		with open(lcmout, "w") as efile:
			pass

		# lcm_seqのパラメータ設定と実行
		params = {}
		if self.msgoff:
			params["type"] ="CIf_"
		else:
			params["type"] ="CIf"

		if self.maxCnt :
			params["U"] = str(self.maxCnt)
		if "minLen" in eArgs:
			params["l"] = str(eArgs["minLen"])
		if 'maxLen' in eArgs:
			params["u"] = str(eArgs["maxLen"])
		if 'gap' in eArgs:
			params["g"] = str(eArgs["gap"])
		if 'win' in eArgs:
			params["G"] = str(eArgs["win"])
		
		params["i"] = self.file
		params["sup"] = str(self.minCnt)
		params["o"] = lcmout


		# lcm_seq実行
		#MCMD::msgLog("#{run}")
		if 'padding' in eArgs and eArgs["padding"] : # padding指定時は、0アイテムを出力しないlcm_seqを実行
			extTake.lcmseq_zero(params)
		else:
			extTake.lcmseq(params)

		# パターンのサポートを計算しCSV出力する
		self.pFile = self.temp.file()
		items=self.db.items

		transl = self.temp.file()
		extTake.lcmtrans(lcmout,"p",transl)


		f = nm.mdelnull(f="pattern",i=transl)
		f <<= nm.mvreplace(vf="pattern",m=items.file,K=items.idFN,f=items.itemFN)
		f <<= nm.msetstr(v=self.db.size, a="total")
		f <<= nm.mcal(c='${count}/${total}',a="support") # サポートの計算
		f <<= nm.mcut(f="pid,pattern,size,count,total,support")
		f <<= nm.msortf(f="support%nr",o=self.pFile)
		f.run()


		if self.outtf :
			# トランザクション毎に出現するシーケンスを書き出す
			#MCMD::msgLog("output tid-patterns ...")
			self.tFile = self.temp.file()

			xxw = tf.file() #Mtemp.new.name
			f=None
			f <<= nm.mcut(f=self.db.idFN,i=self.db.file)
			f <<= nm.muniq(k=self.db.idFN)
			f <<= nm.mnumber(S=0,a="__tid",q=True)
			f <<= nm.msortf(f="__tid",o=xxw)
			f.run()

			translt = self.temp.file()
			extTake.lcmtrans(lcmout,"t",translt)

			f=None
			f <<= nm.msortf(f="__tid",i=translt)
			f <<= nm.mjoin(k="__tid",m=xxw,f=self.db.idFN)
			f <<= nm.mcut(f=self.db.idFN+",pid")
			f <<= nm.msortf(f=self.db.idFN+",pid",o=self.tFile)
			f.run()

		#	self.size = nutil.mrecount(i=self.pFile) # 列挙されたパターンの数
		#MCMD::msgLog("the number of contrast patterns enumerated is #{@size}")

	def output(self,outpath):
		nm.mfldname(q=True,i=self.pFile,o=outpath+"/patterns.csv").run()
		#shutil.move(self.pFile,outpath+"/patterns.csv")
		if self.outtf:
			nm.mfldname(q=True,i=self.tFile,o=outpath+"/tid_pats.csv").run()
			#shutil.move(self.tFile,outpath+"/tid_pats.csv")





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import nysol.mcmd as nm
import nysol.util as nu

import nysol.util.margs as margs
import nysol.util.mtemp as mtemp
from nysol import take as nt


class mpal(object):
	helpMSG="""
# 1.0 initial development: 2016/12/23
# 1.1 dir=x追加: 2016/12/29
# 1.2 jac=等追加: 2017/1/10
# 1.3 ro=追加: 2017/1/15
$version="1.3"
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) トランザクションデータから２アイテム相関ルールを求め、ランクに基づいた類似関係にある相関ルールを選択する

書式) #{$cmd} i= tid= item= [class=] [ro=] [no=] eo= s=|S= [filter=] [lb=] [ub=] [sim=] [rank=] [dir=] [-prune] [T=] [--help]
  i=     : トランザクションデータファイル【必須】
  tid=   : トランザクションID項目名【必須】
  item=  : アイテム項目名【必須】
  ro=    : 出力ルールファイル
  no=    : 出力ファイル(節点)
  eo=    : 出力ファイル(辺:節点ペア)
  s=     : 最小支持度(全トランザクション数に対する割合による指定): 0以上1以下の実数
  S=     : 最小支持度(トランザクション数による指定): 1以上の整数
  filter=: 相関ルールの評価指標(省略可,複数指定可)
           指定できる類似度は以下の4つ(括弧内は値域)。
             J: Jaccard(0..1), P:normalized PMI(-1..0..1), C:Confidence(0..1)
  lb=    : filter=で指定した相関ルール評価指標の下限値
  ub=    : filter=で指定した相関ルール評価指標の上限値
           lb=が省略された場合のデフォルトで0、ub=のデフォルトは1
  例1: sim=P lb=0.5 : normalized PMIが0.5以上1以下の相関ルールを列挙する
  例2: sim=P,C lb=0.5,0.2 ub=,0.8 : 例1に加えて、confidenceが0.2以上0.8以下の相関ルールを列挙する

  sim=   : 列挙された相関ルールを元にして枝を張る条件となる指標を指定する。
           以下に示す4つの相関ルール評価指標が指定できる(デフォルト:S)。
             S:Support, J: Jaccard, P:normalized PMI, C:Confidence
  rank=  : 枝を張る条件で、類似枝の上位何個までを選択するか(デフォルト:3)
  dir=   : 双方向類似関係(b)のみを出力するか、単方向類似関係(m)のみか、両方とも含める(x)かを指定する。(デフォルト:b)
           相関ルールa=>bの類似度をsim(a=>b)としたとき、
           b:(bi-directional) sim(a=>b)およびsim(b=>a)がrank=で指定した順位内である相関ルールのみ選択される。
           m:(mono-directional) 片方向の類似度のみが、指定された順位内である相関ルールが選択される。
           x:(both) bとmの両方共含める。
  以上の3つのパラメータは複数指定することが可能(3つまで)。
  例1: sim=S dir=b rank=3 :
         アイテムaからみてsupport(a=>b)が3位以内で、かつ
         アイテムbからみてsupport(b=>a)も3位以内であるような相関ルールを選択する
  例2: sim=S,C dir=b,m rank=3,1
         例1に加えて、アイテムaから見てconfidenc(a=>b)が3以内、もしくは
         アイテムbから見てconfidenc(b=>a)が3以内であれば、そのような相関ルールも選択する
  -prune : sim=等を複数指定した場合、マルチ枝を単一化する。
           第1優先順位: 双方向>片方向
           第2優先順位: パラメータ位置昇順

  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

入力ファイル形式)
トランザクションIDとアイテムの２項目によるトランザクションデータ。

o=の出力形式)
枝ファイル: simType,simPriority,node1,node2,sim,dir,color
節点ファイル: node,support,frequency,total

例)
$ cat tra.csv
id,item
1,a
1,b
2,a
2,b
3,a
3,b
4,b
4,c
5,a
5,c
6,a
6,c
7,d
7,e
8,d
8,e
9,d
9,e
A,d
A,c
B,e
B,b
C,e
C,a
D,f
D,c
E,f
E,b
F,f
F,a

$ #{$cmd} i=tra1.csv no=node11.csv eo=edge11.csv tid=id S=1 item=item sim=S rank=1 dir=b
$ cat edge11.csv 
simType,simPriority,node1,node2,sim,dir,color
support,0,a,e,0.2727272727,W,FF0000
support,0,d,e,0.2727272727,W,FF0000
support,0,c,f,0.1818181818,W,FF0000
support,0,g,h,0.1818181818,W,FF0000

$ #{$cmd} i=tra2.csv no=node51.csv eo=edge51.csv tid=id S=1 item=item sim=S,C rank=1,1 dir=b,m -prune
$ cat edge51.csv 
simType,simPriority%3n,node1%0,node2%1,sim,dir%2r,color
support,0,a,b,0.2,W,FF0000
confidence,1,c,a,0.4,F,8888FF
support,0,d,e,0.2,W,FF0000
confidence,1,f,a,0.3333333333,F,8888FF
confidence,1,f,b,0.3333333333,F,8888FF
confidence,1,f,c,0.3333333333,F,8888FF

# Copyright(c) NYSOL 2012- All Rights Reserved.
"""

	paramter = {	
		"i": "filename",
		"tid": "fld",
		"item":"fld",
		"ro": "filename",
		"eo": "filename",
		"no": "filename",
		"s" : "float",
		"S":"int",
		"filter":"str",
		"lb":"int",
		"ub":"int",
		"sim": "str",
		"dir" :"str",
		"rank":"fld",
		"prune" : "bool",
		"num": "bool",
		"verbose":"bool",
		"T": "str"
	}
	paramcond = {	
		"hissu":["i","tid","item","eo","no"]
	}
	

	def help():
		print(mpal.helpMSG) 

	def ver():
		print(mpal.versionInfo)

	
	def __param_check_set(self , kwd):
		for k,v in kwd.items():
			if not k in mpal.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )
			#型チェック入れる

		self.iFile   = kwd["i"]
		self.idFN    = kwd["tid"]
		self.itemFN  = kwd["item"]
		self.onFile  = kwd["no"]
		self.oeFile  = kwd["eo"]

		self.orFile  =  kwd["ro"] if "ro" in kwd else None
				
		self.sp1    = kwd["s"] if "s" in kwd else None
		self.sp2    = kwd["S"] if "S" in kwd else None


		self.numtp = kwd["num"]   if "num"   in kwd else False
		self.prune = kwd["prune"] if "prune" in kwd else False 

		self.dir   = kwd["dir"].split(",")  if "dir"  in kwd else ["b"]
		self.rank  = kwd["rank"].split(",") if "rank" in kwd else ["3"]

		_sim    = kwd["sim"].split(",")    if "sim"  in kwd else ["S"]

		_filter = kwd["filter"].split(",") if "filter" in kwd else [] 
		_ub     = kwd["ub"].split(",")     if "ub" in kwd else [] 
		_lb     = kwd["lb"].split(",")     if "lb" in kwd else [] 

		self.filter=[None]*3
		self.lb=["0"]*3
		self.ub=["1"]*3

		self.sim=[]

		self.filterSize = len(_filter)

		# chicking parameter for filter=
		if len(_filter)>0:
			if len(_filter) != len(set(_filter)):
				raise Exception("filter= cannot take same values")
					
			if len(_filter) > 3:
				raise Exception("flter=,lb=,ub= takes parameters with 0<size<=3")
				
			for s in _filter:
				if s!="J" and s!="P" and s!="C" :
					raise Exception("filter= takes J, P or C")
			
			# dictに
			for i in range(len(_filter)):
				if _filter[i] == "J":
					self.filter[i] = "jaccard"
				elif _filter[i] == "P":
					self.filter[i] = "PMI"
				elif _filter[i] == "C":
					self.filter[i] = "confidence"
				else:
					raise Exception("filter= takes J, P or C")
				
				if len(_lb) > i :
					self.lb[i] = _lb[i]

				if len(_ub) > i :
					self.ub[i] = _ub[i]


		# chicking parameter for sim=
		if len(_sim) != len(set(_sim)):
			raise Exception("sim= cannot take same values")

		if len(_sim)>3 :
			raise Exception("sim=,dir=,rank= takes parameters with 0<size<=3")

		if not ( len(_sim)==len(self.dir) and len(self.dir) == len(self.rank)):
			raise Exception("sim=,dir=,rank= must take same size of parameters")

		for s in _sim:

			if s=="S":
				self.sim.append("support")
			elif s=="J":
				self.sim.append("jaccard")
			elif s=="P":
				self.sim.append("PMI")
			elif s=="C":
				self.sim.append("confidence")
			else:
				raise Exception("sim= takes S, J, P or C")

		for s in self.dir:
			if s!="b" and s!="m" and s!="x" :
				raise Exception("dir= takes b, m, x")

		for s in self.rank:
			if int(s) < 0 :
				raise Exception("rank= takes positive integer")

	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 


	def __init__(self,**kwd):

		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)



	# ============
	# entry point
	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		temp=mtemp.Mtemp()

		### mtra2gc
		xxsimgN=temp.file()
		xxsimgE=temp.file()
		xxsimgE0=temp.file()

		param = {}
		param["i"] = self.iFile
		if self.idFN:
			param["tid"]  = self.idFN
		if self.itemFN:
			param["item"] = self.itemFN
		if self.sp1:
			param["s"] = self.sp1
		if self.sp2:
			param["S"] = self.sp2

		#####################
		# 異なる向きのconfidenceを列挙するためにsim=C th=0として双方向列挙しておく
		# 出力データは倍になるが、mfriendsで-directedとすることで元が取れている
		param["sim"]="C"
		param["th"]= "0"

		param["node_support"] = True
		if self.numtp :
			param["num"] = True
		param["no"] = xxsimgN
		param["eo"] = xxsimgE0

		nt.mtra2gc(**param).run(**kw_args)

		f=nm.readcsv(xxsimgE0)
		for i in range(self.filterSize):
			f <<= nm.mselnum(f=self.filter[i],c="[%s,%s]"%(self.lb[i],self.ub[i]))
		f <<= nm.writecsv(xxsimgE)
		f.run()


		### mfrirends
		xxfriends=temp.file()
		xxfriendE=temp.file()
		xxw=temp.file()
		xxf=temp.file()
		xxff=temp.file()
		xxor=temp.file()

		if not os.path.isdir(xxfriends) :
			os.makedirs(xxfriends)
		col=[ ["FF000080","FF888880"], ["0000FF80","8888FF80"], ["00FF0080","88FF8880"]]

		for i in range(len(self.sim)):
			paramf ={}
			paramf["ei"] = xxsimgE
			paramf["ni"] = xxsimgN
			paramf["ef"] = "node1,node2"
			paramf["nf"] = "node"
			paramf["eo"] = xxfriendE
			paramf["no"] = xxfriends+"/n_"+str(i)
			paramf["sim"]  = self.sim[i]
			paramf["dir"]  = self.dir[i]
			paramf["rank"] = self.rank[i]
			paramf["directed"] = True

			nt.mfriends(**paramf).run(**kw_args)
			
			frec2 = nm.mfsort(f="node1,node2",i=xxfriendE)
			frec2 <<= nm.msummary(k="node1,node2",f=self.sim[i],c="count,mean")
			frec2 <<= nm.mselstr(f="count",v=2)
			# node1%0,node2%1,fld,count,mean
			# a,b,support,2,0.1818181818
			# a,d,support,2,0.1818181818

			f = nm.mjoin(k="node1,node2",K="node1,node2",m=frec2,f="mean:s1",n=True,i=xxfriendE)
			f <<= nm.mjoin(k="node2,node1",K="node1,node2",m=frec2,f="mean:s2",n=True)
			# 1) xxrecs2でsimをjoinできない(s1,s2共にnull)ということは、それは片方向枝なので"F"をつける
			# 2) 双方向枝a->b,b->aのうちa->bのみ(s1がnullでない)に"W"の印をつける。
			# 3) それ以外の枝は"D"として削除
			f <<= nm.mcal(c='if(isnull($s{s1}),if(isnull($s{s2}),\"F\",\"D\"),\"W\")',a="dir")
			f <<= nm.mselstr(f="dir",v="D",r=True)
			f <<= nm.mcal(c='if($s{dir}==\"W\",$s{s1},$s{%s})'%(self.sim[i]),a="sim")
			f <<= nm.mchgstr(f="dir:color",c='W:%s,F:%s'%(col[i][0],col[i][1]),A=True)
			f <<= nm.msetstr(v=[self.sim[i],str(i)],a="simType,simPriority")
			f <<= nm.mcut(f="simType,simPriority,node1,node2,sim,dir,color",o=xxfriends+"/e_"+str(i))
			f.run()
			# node1%1,node2%0,simType,sim,dir,color
			# b,a,jaccard,0.3333333333,F,8888FF
			# j,c,jaccard,0.3333333333,F,8888FF
			# b,d,jaccard,0.3333333333,F,8888FF
			# a,e,jaccard,0.5,W,0000FF
			# d,e,jaccard,0.5,W,0000FF


		# rule fileの出力
		if self.orFile:
			mmm=nm.mcat(i=xxfriends+"/e_*").muniq(k="node1,node2")
			nm.mcommon(k="node1,node2",i=xxsimgE , m=mmm, o=self.orFile).run()

		# マルチ枝の単一化(W優先,パラメータ位置優先)
		if self.prune:
			"""
			# 双方向と片方向に分割
			nm.mcat(i=xxfriends+"/e_*").mselstr(f="dir",v="W",o=xxw,u=xxf).run()
			# 片方向のみの枝を選択
			f =   nm.mcommon(k="node1,node2",K="node1,node2",r=True,m=xxw,i=xxf)
			f <<= nm.mcommon(k="node1,node2",K="node2,node1",r=True,m=xxw,o=xxff)
			f.run()
			f = nm.mcat(i=xxw+","+xxff).mbest(k="node1,node2",s="dir%r,simPriority%n",o=self.oeFile).run()
			"""
			#これだめ
			fo = nm.mcat(i=xxfriends+"/e_*").mselstr(f="dir",v="W")
			fu = fo.redirect("u") # これは再考
			fu <<= nm.mcommon(k="node1,node2",K="node1,node2",r=True,m=fo)
			fu <<= nm.mcommon(k="node1,node2",K="node2,node1",r=True,m=fo)
			#f  =   nm.m2cat()
			f  = nm.mbest(i=[fo,fu],k="node1,node2",s="dir%r,simPriority%n",o=self.oeFile)

			f.run()



		else:
			nm.mcat(i=xxfriends+"/e_*",o=self.oeFile).run()

		nm.mcat(i=xxfriends+"/n_0",o=self.onFile).run()

		nu.mmsg.endLog(self.__cmdline())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt

args=margs.Margs(sys.argv,"ei=,ef=,eo=,th=,sim=,th2=,sim2=,kn=,kn2=,sup=,iter=,log=","ei=,ef=,th=")
nt.mbipolish(**args.kvmap()).run(msg="on")






#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt


args=margs.Margs(sys.argv,"ei=,ef=,ni=,nf=,-all,o=,l=,u=,log=","ei=,ef=")
nt.mclique(**(args.kvmap())).run(msg="on")




# -*- coding: utf-8 -*-
import sys
import re

# これは簡易版
def dicisionPosSub(mlist,iolist,baselist,dsppos,y,counter,basecont):

	for i in baselist:
		if dsppos[i] != None:
			continue
		dsppos[i] = [counter[y],y]
		counter[y]+=1

		newlist = []
		for x in iolist[i][0]:
			newlist.append(x[0])

		if not y-1 in counter.keys():
			counter[y-1]=basecont

		dicisionPosSub(mlist,iolist,newlist,dsppos,y-1,counter,basecont)

		newlist = []
		for x in iolist[i][1]:
			newlist.append(x[0])

		if not y+1 in counter.keys():
			counter[y+1]=basecont

		dicisionPosSub(mlist,iolist,newlist,dsppos,y+1,counter,basecont)


	
def dicisionPos(mlist,iolist):

	startpos = set() 
	for i , mm in enumerate(mlist):
		if mm[0]=="ifile" or mm[0]=="file" :
			startpos.add(i)


	for i , mm in enumerate(iolist):
		if len(mm[0]) == 0 :
			startpos.add(i)

	dsppos   = [None]*len(mlist)
	counter ={0:0}
	y=0
	basecont=0
	maxlen=0
	minpos =0;

	for x in list(startpos):
		dicisionPosSub(mlist,iolist,[x],dsppos,y,counter,basecont)
		basecont=max(counter.values())+1

		if minpos > min(counter.keys()):
			minpos = min(counter.keys())

		if maxlen < len(counter):
			maxlen = len(counter)

		counter = {0:basecont}
	
	if minpos < 0 :
		for val in dsppos:
			val[1]=val[1]-minpos

	return dsppos , maxlen ,basecont


def filenameEXTRACT(mlist,iolist,linklist):

	addmod =[]
	addfnod ={}
	addlnod ={}

	maxCnt = len(mlist)
	for i, mm in enumerate(mlist):

		if len(mm[2]) != 0: # in

			for k,v in mm[2].items():
			
				if isinstance(v, list):
					vid = id(v)
					if vid in addlnod:
						newpos = addlnod[vid]
					else:
						addlnod[vid]=maxCnt
						iolist.append([[], []])
						addmod.append(['list',["ilist_"+str(i)],{},{},''])
						newpos = maxCnt
						maxCnt+=1
				else:
					if v in addfnod:
						newpos = addfnod[v]
					else:
						addfnod[v]=maxCnt
						iolist.append([[], []])
						addmod.append(['file',[v],{},{},''])
						newpos = maxCnt
						maxCnt+=1
						
				iolist[i][0].append([newpos,k])
				iolist[newpos][1].append([i,'o'])
				linklist.append([["o",newpos],[k,i]])

		if len(mm[3]) != 0: # out
		
			for k,v in mm[3].items():

				if isinstance(v, list):

					vid = id(v)

					if vid in addlnod:

						newpos = addlnod[vid]

					else:
						addlnod[vid]=maxCnt
						iolist.append([[], []])
						addmod.append(['list',["olist_"+str(i)],{},{},''])
						newpos = maxCnt
						maxCnt+=1

				else:
				
					if v in addfnod:
						newpos = addfnod[v] 
					else:
						addfnod[v]=maxCnt
						iolist.append([[], []])
						addmod.append(['file',[v],{},{},''])
						newpos = maxCnt
						maxCnt+=1
				

				iolist[i][1].append([newpos,k])
				iolist[newpos][0].append([i,"i"])
				linklist.append([[k,i],["i",newpos]])
				

		mlist[i][2] ={}
		
	mlist.extend(addmod)

def filenameEXTRACTX(mlist,iolist,linklist):

	addmod =[]
	addfnod ={}
	addlnod ={}

	maxCnt = len(mlist)
	for i, mm in enumerate(mlist):

		if len(mm[2]) != 0: # in

			for k,v in mm[2].items():
			
				if isinstance(v, list):
					vid = id(v)
					if vid in addlnod:
						newpos = addlnod[vid]
					else:
						addlnod[vid]=maxCnt
						iolist.append([[], []])
						addmod.append(['list',[v],{},{},''])
						newpos = maxCnt
						maxCnt+=1
				else:
					if v in addfnod:
						newpos = addfnod[v]
					else:
						addfnod[v]=maxCnt
						iolist.append([[], []])
						addmod.append(['file',[v],{},{},''])
						newpos = maxCnt
						maxCnt+=1
						
				iolist[i][0].append([newpos,k])
				iolist[newpos][1].append([i,'o'])
				linklist.append([["o",newpos],[k,i]])

		if len(mm[3]) != 0: # out
		
			for k,v in mm[3].items():

				if isinstance(v, list):

					vid = id(v)

					if vid in addlnod:

						newpos = addlnod[vid]

					else:
						addlnod[vid]=maxCnt
						iolist.append([[], []])
						addmod.append(['list',[v],{},{},''])
						newpos = maxCnt
						maxCnt+=1

				else:
				
					if v in addfnod:
						newpos = addfnod[v] 
					else:
						addfnod[v]=maxCnt
						iolist.append([[], []])
						addmod.append(['file',[v],{},{},''])
						newpos = maxCnt
						maxCnt+=1
				

				iolist[i][1].append([newpos,k])
				iolist[newpos][0].append([i,"i"])
				linklist.append([[k,i],["i",newpos]])
				

		mlist[i][2] ={}
		
	mlist.extend(addmod)



def chageSVG(mlist,iolist,linklist,fname=None):
	"""
	OUTPUT FLOW SVG HTML  
	"""

	filenameEXTRACT(mlist,iolist,linklist)
	dsppos,ymax,xmax = dicisionPos(mlist,iolist)

	if fname == None:
		f=sys.stdout
	else:
		f=open(fname, 'w')
	
	f.write("<svg height='"+  str(ymax*60) + "' width='" + str(xmax*60) + "'>\n")
	f.write("<defs>\n")
	f.write("<marker id='endmrk' markerUnits='strokeWidth' markerWidth='3' markerHeight='3' viewBox='0 0 10 10' refX='5' refY='5' orient='auto'>\n")
	f.write("<polygon points='0,0 5,5 0,10 10,5 ' fill='black'/>\n")
	f.write("</marker>\n")
	f.write("</defs>\n")


	for i , mm in enumerate(dsppos):
		
		modobj = mlist[i]
		x,y = mm

		f.write("<g>\n")

		if modobj[0] == "file" :
			flist =[]
			for full in modobj[1] :
				flist.append(full.split("/")[-1])
			titlestr = ",".join(flist)

		elif modobj[0] == "cmd":
			import re
			titlestr = re.sub(r'^cmdstr=(.*)',r'\1'," ".join(modobj[1])).replace('"', '\\"')

		else:
			titlestr = modobj[0] + " "+ " ".join(modobj[1]).replace('"', '\\"') 

		if  modobj[4] == "" :
			f.write("<title>" + titlestr + "</title>\n" )
		else:
			f.write("<title>" + titlestr+ "@" + modobj[4] + "</title>\n" )

		if modobj[0] == "cmd":

			mstr = "<circle cx='" + str(x*60+20) + "' cy='" + str(y*60+20) + "' r='20' stroke='blue' fill='white' stroke-width='1'/>\n"
			namevals =  titlestr.split()
			if len(namevals)>0:
				nameval = namevals[0]
			else:
				nameval = modobj[0]
			mstr += "<text x='" + str(x*60) + "' y='" + str(y*60+20) + "' fill='gray'>\n" + nameval + "\n</text>\n"

		elif modobj[0] == "file" or modobj[0] == "list" :

			mstr = "<rect x='" + str(x*60) + "' y='" + str(y*60+5) + "' width='40' height='40' stroke='blue' fill='white' stroke-width='1'/>\n"
			mstr += "<text x='" + str(x*60) + "' y='" + str(y*60+20) + "' fill='black'>\n" + modobj[0] + "\n</text>\n"

		else:

			mstr = "<circle cx='" + str(x*60+20) + "' cy='" + str(y*60+20) + "' r='20' stroke='blue' fill='white' stroke-width='1'/>\n"
			mstr += "<text x='" + str(x*60) + "' y='" + str(y*60+20) + "' fill='black'>\n" + modobj[0] +  "\n</text>\n"

		f.write(mstr) 
		f.write("</g>\n")

	for fr , to in linklist:
		frNo = fr[1] 
		toNo = to[1] 
		frTp = fr[0] 
		toTp = to[0] 
		 
		frX , frY = dsppos[frNo]
		toX , toY = dsppos[toNo]
		x = toX-frX
		y = toY-frY
		z = ((x ** 2) + (y ** 2)) ** 0.5
		
		xsub = 20.0 * x / z
		ysub = 20.0 * y / z

		f.write("<g>\n")
		f.write("<title>" + frTp + " => " + toTp + "</title>\n" )
		f.write("<line x1='" + str(20+frX*60+xsub) + "' y1='" + str(20+frY*60+ysub) + "' x2='" + str(20+toX*60-xsub) + "' y2='" + str(20+toY*60-ysub) + "' stroke='black' stroke-width='5' marker-end='url(#endmrk)'/>\n")
		f.write("</g>\n")


	f.write("</svg>\n")

	f.close()



def chageSCP(mlist,iolist,linklist,fname=None,msgF=False):
	"""
	OUTPUT FLOW SCP 
	"""
	filenameEXTRACTX(mlist,iolist,linklist)
	dsppos,ymax,xmax = dicisionPos(mlist,iolist)

	# 順位LIST生成 : file listは先頭
	rankL = [ []  for _ in range(ymax+1)]

	for i,pinfo in enumerate(dsppos):
		if mlist[i][0]=="file" or mlist[i][0]=="list":
			rankL[0].append(i)
		else:	
			rankL[pinfo[1]+1].append(i)

	# fallten			
	rankList = [ i for lls in rankL for i in lls]
		
	if fname == None:
		f=sys.stdout
	else:
		f=open(fname, 'w')

	scpStr = []

	#norma1:0
	#file:1 
	#cdm:2 
	runNum=[]
	for i in rankList:
		modobj = mlist[i]
		#for i,modobj in enumerate(mlist):

		if modobj[0] == "file" :
			flist =[]
			for full in modobj[1] :
				flist.append(full.split("/")[-1])
			titlestr = [ "file" , ",".join(flist) ]

		elif modobj[0] == "cmd":
			#titlestr = [ "cmd", re.sub(r'^cmdstr=(.*)',r'\1'," ".join(modobj[1])).replace('"', '\\"')]
			comstr = re.sub(r'^cmdstr=(.*)',r'\1'," ".join(modobj[1])).replace('"', '\\"')
			titlestr = [ "cmd", 'cmdstr="' + comstr +'"']


		elif modobj[0] == "list":
			titlestr = [ "list" , str(modobj[1][0]) ]

		else:
			plist=[]
			for para in modobj[1]:
				val = para.split("=",1)
				if len(val)==1 :
					if val[0][0] == '-':
						plist.append(re.sub(r'^-', "", val[0] )+"=True")
					else:
						plist.append(val[0])
				else:
					plist.append(val[0]+"="+"'" + val[1] + "'")

			titlestr = [ modobj[0] , ",".join(plist).replace('"', '\\"') ]

		if titlestr[0] == "file" :

			scpStr.append("f_%d = '%s'\n"%(i,titlestr[1]))

		elif titlestr[0] == "list":

			scpStr.append("f_%d = %s\n"%(i,titlestr[1]))

		else:
			# input
			infn = {}
			istr=""
			if len(iolist[i][0])>0:
				for ilist in iolist[i][0]:
					if not ( ilist[1] in infn ):
						infn[ilist[1]] = [] 
						
					# 接続先確認
					eFlg=False
					for toid , dir  in iolist[ilist[0]][1]:
						if toid == i and dir != "o":
							infn[ilist[1]].append("f_%d.redirect('%s')"%(ilist[0],dir)) 
							eFlg =True

					if not eFlg :
						infn[ilist[1]].append("f_%d"%(ilist[0])) 
				
				
				istrsub = []
				for  k,v  in infn.items():
					ival = ""
					if len(v) == 1 :
						ival = v[0]
					elif len(v) > 1 :
						ival = "[" + ",".join(v) + "]"
					
					if titlestr[0] == "readlist":
						istrsub.append(ival)
					else:
						istrsub.append( k + "=" + ival)
					
				istr = ','.join(istrsub) 

			# output
			outfn = {}
			ostr=""

			rokFlg= False
			if len(iolist[i][1])>0:
				for olist in iolist[i][1]:
					if mlist[olist[0]][0] == "file" or mlist[olist[0]][0] == "list" :
						if not (olist[1] in outfn ):
							outfn[olist[1]] = []
						outfn[olist[1]].append("f_%d"%(olist[0])) 

				for  k,v  in outfn.items():
					if k=="o": # should coompare default 
						rokFlg = True

					oval = ""
					if len(v) == 1 :
						oval = v[0]
					elif len(v) > 1 :
						oval = "[" + ",".join(v) + "]"

					ostr += ( k + "=" + oval)

			pplist=[]
			if istr != "":
				pplist.append(istr)
			if ostr != "":
				pplist.append(ostr)
				if rokFlg :
					runNum.append("f_%d"%(i))

			if titlestr[1] != "":
				pplist.append(titlestr[1])
				
			scpStr.append( "f_%d=nm.%s(%s)\n"%(i,titlestr[0],",".join(pplist)) )

	f.write("import nysol.mcmd as nm\n")
	for scp in scpStr:
		f.write(scp)
	if msgF:
		f.write("nm.runs([%s],msg='on')\n"%(','.join(runNum)))	
	else:
		f.write("nm.runs([%s])\n"%(','.join(runNum)))

	f.close()

def chageSVG_D3(mlist,iolist,linklist,fname=None):
	"""
		OUTPUT FLOW SVG HTML USING D3.V3 
	"""
	filenameEXTRACT(mlist,iolist,linklist)

	dsppos,ymax,xmax = dicisionPos(mlist,iolist)


	if fname == None:
		f=sys.stdout
	else:
		f=open(fname, 'w')
	
	f.write("<html>\n")
	f.write("<head>\n")
	
	f.write("<script src='http://d3js.org/d3.v3.min.js' charset='utf-8'></script>\n")
	f.write("<script>\n")
	f.write("var NodeDATA=[")

	mlastNo = len(dsppos)
	for i , mm in enumerate(dsppos):
		
		modobj = mlist[i]

		x,y = mm

		if modobj[0] == "file":
			flist =[]
			for full in modobj[1] :
				flist.append(full.split("/")[-1])
			titlestr = ",".join(flist)

		elif modobj[0] == "cmd":
			import re
			titlestr = re.sub(r'^cmdstr=(.*)',r'\1'," ".join(modobj[1]))

		else:
			titlestr = modobj[0] + " "+ " ".join(modobj[1])

		if  modobj[4] == "" :
			f.write("{ title:\"%s\"," % (titlestr.replace('"', '\\"')) ) 
		else:
			f.write("{ title:\"%s @ %s\"," % (titlestr.replace('"', '\\"'),modobj[4]) ) 

		if modobj[0] == "cmd":
			namevals =  titlestr.split()
			if len(namevals)>0:
				nameval = namevals[0]
			else:
				nameval = modobj[0]
			f.write(" x:%d , y:%d , name:\"%s\" ,tp:\"excmd\"}" % (x*60+20,y*60+20, nameval) ) 

		elif modobj[0] == "file" :
			f.write(" x:%d , y:%d , name:\"%s\" ,tp:\"file\"}" % (x*60+20,y*60+20, modobj[0])) 
		elif modobj[0] == "list":
			f.write(" x:%d , y:%d , name:\"%s\" ,tp:\"list\"}" % (x*60+20,y*60+20, modobj[0])) 
		else:
			if modobj[5] == True:
				f.write(" x:%d , y:%d , name:\"%s\", tp:\"sysadd\"}" % (x*60+20,y*60+20, modobj[0])) 
			else:
				f.write(" x:%d , y:%d , name:\"%s\", tp:\"mod\"}" % (x*60+20,y*60+20, modobj[0])) 
				

		if mlastNo==i+1 :
			f.write("]\n")
		else:
			f.write(",\n")

	elastNo = len(linklist)
	
	if elastNo == 0 :
		f.write("var EdgeDATA=[];\n")
		f.write("var LinkLIST=[];\n")

	else:
		f.write("var EdgeDATA=[")
		linklist_n2e=[None]*mlastNo

		for i , frto in enumerate(linklist):
			fr,to = frto
			frNo = fr[1] 
			toNo = to[1] 
			frTp = fr[0] 
			toTp = to[0] 
		 
			frX , frY = dsppos[frNo]
			toX , toY = dsppos[toNo]

			if linklist_n2e[frNo] == None:
				linklist_n2e[frNo] =[[],[]]

			linklist_n2e[frNo][0].append(str(i))

			if linklist_n2e[toNo] == None:
				linklist_n2e[toNo] =[[],[]]

			linklist_n2e[toNo][1].append(str(i))

			f.write("{ title:\"%s => %s \"," % (frTp,toTp))
	#		f.write(" x1:%d,y1:%d,x2:%d,y2:%d }" % (frX*60+20,frY*60+40,toX*60+20,toY*60+0))
			f.write(" x1:%d,y1:%d,x2:%d,y2:%d }" % (frX*60+20,frY*60+20,toX*60+20,toY*60+20))

			if elastNo==i+1 :
				f.write("]\n")
			else:
				f.write(",\n")

		n2elastNo = len(linklist_n2e)
		f.write("var LinkLIST=[")
		for i , n2elist in enumerate(linklist_n2e):
			if n2elist == None:
				continue
			f.write("[[%s],[%s]]"%(",".join(n2elist[0]),",".join(n2elist[1])))
			if n2elastNo ==i+1 :
				f.write("]\n")
			else:
				f.write(",\n")

	f.write("</script>\n")
	f.write("</head>")
	f.write("<body>")
	f.write("<svg id='flowDspArea' height='%d' width='%d'>\n" % (ymax*60*2 ,xmax*60*2))
	f.write("<defs>\n")
	f.write("<marker id='endmrk' markerUnits='strokeWidth' markerWidth='3' markerHeight='3' viewBox='0 0 10 10' refX='5' refY='5' orient='auto'>\n")
	f.write("<polygon points='0,0 5,5 0,10 10,5 ' fill='black'/>\n")
	f.write("</marker>\n")
	f.write("</defs>\n")
	f.write("</svg>\n")
	scp = """\
	<script>
	svgGroup = d3.select('#flowDspArea');
	
	node_g = svgGroup.selectAll('g .node').data(NodeDATA);
	edge_g = svgGroup.selectAll('g .edge').data(EdgeDATA);
	// 移動処理用
	var drag = d3.behavior.drag()
	drag.on('drag', dragMove);
	function dragMove(d,i) {
		d.x += d3.event.dx
  	d.y += d3.event.dy
   	d3.select(this)
   		.attr('transform','translate('+d.x+','+d.y+')')

		for(var j=0 ; j<LinkLIST[i][0].length;j++){
			EdgeDATA[LinkLIST[i][0][j]].x1 += d3.event.dx
			EdgeDATA[LinkLIST[i][0][j]].y1 += d3.event.dy
	   	d3.select('#edgeP-'+LinkLIST[i][0][j])
				.attr('x1',function(d) {
					return d.x1 + ( 20.0 * (d.x2-d.x1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
				.attr('x2',function(d) {
					return d.x2 - ( 20.0 * (d.x2-d.x1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
				.attr('y1',function(d) {
					return d.y1 + ( 20.0 * (d.y2-d.y1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
				.attr('y2',function(d) { 
					return d.y2 - ( 20.0 * (d.y2-d.y1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
		}
		for(var j=0 ; j<LinkLIST[i][1].length;j++){
			EdgeDATA[LinkLIST[i][1][j]].x2 += d3.event.dx
			EdgeDATA[LinkLIST[i][1][j]].y2 += d3.event.dy
	   	d3.select('#edgeP-'+LinkLIST[i][1][j])
				.attr('x1',function(d) {
					return d.x1 + ( 20.0 * (d.x2-d.x1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
				.attr('x2',function(d) {
					return d.x2 - ( 20.0 * (d.x2-d.x1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
				.attr('y1',function(d) {
					return d.y1 + ( 20.0 * (d.y2-d.y1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
				.attr('y2',function(d) { 
					return d.y2 - ( 20.0 * (d.y2-d.y1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
				})
		}
	}
 	node_g2 = node_g.enter().append('g')
		.attr('class', 'node')
		.attr('id', function (d,i) {return 'node-' + i;})
		.attr('transform',function (d) { return 'translate('+d.x+','+d.y+')'})
    .call(drag)		

	node_g2.append('title')
			.text(function(d) { return d.title})

	node_g2
		.filter(function(d) { return d.tp != "file" && d.tp != "list" && d.tp != "sysadd"})
		.append('circle')
		.attr('r',20)
		.attr('stroke','blue')
		.attr('fill','white')
		.attr('stroke-width',1)

	node_g2
		.filter(function(d) { return d.tp == "sysadd"})
		.append('circle')
		.attr('r',20)
		.attr('stroke','gray')
		.attr('fill','white')
		.attr('stroke-width',1)

	node_g2
		.filter(function(d) { return d.tp == "file" || d.tp == "list"  })
		.append('rect')
		.attr('x',-20)
		.attr('y',-15)
		.attr('width',40)
		.attr('height',30)
		.attr('stroke','blue')
		.attr('fill','white')
		.attr('stroke-width',1)

	node_g2.append('text')
		.attr('x',function(d) { return -20})
		.attr('fill',function(d) { 
			if ( d.tp == 'excmd' || d.tp == "sysadd" ){ return 'gray'; } 
			else { return 'black'; }
		})
		.text(function(d) { return d.name})

 	edge_g2 = edge_g.enter().append('g')
		.attr('class', 'edge')
		.attr('id', function (d,i) {return 'edge-' + i;})


	edge_g2.append('line')
		.attr('id', function (d,i) {return 'edgeP-' + i;})
		.attr('x1',function(d) {
			return d.x1 + ( 20.0 * (d.x2-d.x1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
		})
		.attr('x2',function(d) {
			return d.x2 - ( 20.0 * (d.x2-d.x1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
		})
		.attr('y1',function(d) {
			return d.y1 + ( 20.0 * (d.y2-d.y1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
		})
		.attr('y2',function(d) { 
			return d.y2 - ( 20.0 * (d.y2-d.y1) / (Math.pow(Math.pow(d.x2-d.x1,2)+Math.pow(d.y2-d.y1,2),0.5)))
		})
		.attr('stroke','black')
		.attr('stroke-width','5')
		.attr('marker-end','url(#endmrk)')

	edge_g2.append('title')
			.text(function(d) { return d.title})

	</script>
	"""
	f.write(scp)
	f.write("</body>\n")
	f.write("</html>\n")
	f.close()




